@@ -19,6 +19,7 @@
  */
 
 #include "box_db_table.h"
+#include "data_structure/glomconversions.h"
 #include <sstream>
 
 Box_DB_Table::Box_DB_Table()
@@ -129,11 +130,13 @@ void Box_DB_Table::fill_fields()
 */
 }
 
-bool Box_DB_Table::record_delete(const Glib::ustring& strPrimaryKeyValue)
+bool Box_DB_Table::record_delete(const Gnome::Gda::Value& primary_key_value)
 {
-  if(strPrimaryKeyValue.size())
+  Field field_primary_key;
+  bool test = get_field_primary_key(field_primary_key);
+  if(test && !GlomConversions::value_is_empty(primary_key_value))
   {
-    return Query_execute( "DELETE FROM " + m_strTableName + " WHERE " + get_table_name() + "." + get_PrimaryKey_Name() + " = " + strPrimaryKeyValue );
+    return Query_execute( "DELETE FROM " + m_strTableName + " WHERE " + m_strTableName + "." + field_primary_key.get_name() + " = " + field_primary_key.sql(primary_key_value) );
   }
   else
   {
@@ -141,14 +144,19 @@ bool Box_DB_Table::record_delete(const Glib::ustring& strPrimaryKeyValue)
   }
 }
 
-Glib::RefPtr<Gnome::Gda::DataModel> Box_DB_Table::record_new(Glib::ustring strPrimaryKeyValue /* = "" */)
+Glib::RefPtr<Gnome::Gda::DataModel> Box_DB_Table::record_new(Gnome::Gda::Value primary_key_value)
 {
-  if(strPrimaryKeyValue.size() == 0)
-  {
-    strPrimaryKeyValue = "0"; //To auto-increment.
-  }
+g_warning("Box_DB_Table::record_new()");
+//TODO: I don't think this method is ever called. murrayc.
 
-  return Query_execute( "INSERT INTO " + m_strTableName + " (" + get_PrimaryKey_Name() + ") VALUES (" + strPrimaryKeyValue + ")" );
+  Field field_primary_key;
+  bool test = get_field_primary_key(field_primary_key);
+  if(test && !GlomConversions::value_is_empty(primary_key_value))
+  {   
+    return Query_execute( "INSERT INTO " + m_strTableName + " (" + get_PrimaryKey_Name() + ") VALUES (" + field_primary_key.sql(primary_key_value) + ")" );
+  }
+  else
+    return Glib::RefPtr<Gnome::Gda::DataModel>();
 }
 
 guint Box_DB_Table::get_Entered_Field_count() const
@@ -222,6 +230,36 @@ bool Box_DB_Table::get_field_primary_key(const type_vecFields& fields, guint& fi
   return false; //Not found.
 }
 
+bool Box_DB_Table::get_field_primary_key(Field& field) const
+{
+ return get_field_primary_key(m_Fields, field);
+}
+
+//static:
+bool Box_DB_Table::get_field_primary_key(const type_vecFields& fields, Field& field)
+{
+  //Initialize input parameter:
+  field = Field();
+
+  //TODO_performance: Cache the primary key?
+  guint col = 0;
+  guint cols_count = fields.size();
+  while(col < cols_count)
+  {
+    if(fields[col].get_field_info().get_primary_key())
+    {
+      field = fields[col];
+      return true;
+    }
+    else
+    {
+      ++col;
+    }
+  }
+
+  return false; //Not found.
+}
+
 bool Box_DB_Table::get_field_index(const Glib::ustring& name, guint& field_index) const
 {
   //Initialize input parameter:
@@ -298,185 +336,23 @@ Box_DB_Table::type_map_valuetypes Box_DB_Table::get_field_type_names()
   return mapTypes;
 }
 
-Field Box_DB_Table::get_fields_for_table_one_field(const Glib::ustring& table_name, const Glib::ustring& field_name) const
+bool Box_DB_Table::get_fields_for_table_one_field(const Glib::ustring& table_name, const Glib::ustring& field_name, Field& field) const
 {
-  Field result;
+  //Initialize output parameter:
+  Field result = Field();
 
   type_vecFields fields =  get_fields_for_table(table_name);
   type_vecFields::iterator iter = std::find_if(fields.begin(), fields.end(), predicate_FieldHasName<Field>(field_name));
   if(iter != fields.end()) //TODO: Handle error?
-    result = *iter;  
-
-  return result;
-}
-
-Box_DB_Table::type_vecFields Box_DB_Table::get_fields_for_table(const Glib::ustring& table_name) const
-{
-  //Get field definitions from the database, because not all field information is stored in the document (for instance, whether it is a primary key)
-  type_vecFields fieldsDatabase = get_fields_for_table_from_database(table_name);
-
-  const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
-  if(!pDoc)
-    return fieldsDatabase; //This should never happen.
-  else
   {
-    type_vecFields fieldsDocument = pDoc->get_table_fields(m_strTableName);
-
-    //Look at each field in the database:
-    for(type_vecFields::iterator iter = fieldsDocument.begin(); iter != fieldsDocument.end(); ++iter)
-    {
-      Glib::ustring field_name = iter->get_name();
-
-      //Get the field info from the database:
-      //This is in the document as well, but it _might_ have changed.
-      type_vecFields::const_iterator iterFindDatabase = std::find_if(fieldsDatabase.begin(), fieldsDatabase.end(), predicate_FieldHasName<Field>(field_name));
-
-      //TODO: What shall we do about fields that don't exist in the database anymore?
-      if(iterFindDatabase != fieldsDatabase.end() )
-      {
-        Gnome::Gda::FieldAttributes field_info_document = iter->get_field_info();
-         
-        //Update the Field information that _might_ have changed in the database.
-        Gnome::Gda::FieldAttributes field_info = iterFindDatabase->get_field_info();
-
-        //libgda does not tell us whether the field is auto_incremented, so we need to get that from the document.
-        field_info.set_auto_increment( field_info_document.get_auto_increment() );
-
-        //libgda does not tell us whether the field is auto_incremented, so we need to get that from the document.
-        field_info.set_primary_key( field_info_document.get_primary_key() );
-        
-        //libgda does yet tell us correct default_value information so we need to get that from the document.
-        field_info.set_default_value( field_info_document.get_default_value() );
-
-        iter->set_field_info(field_info);
-      }
-    }
-
-    //Add any fields that are in the database, but not in the document:
-    for(type_vecFields::iterator iter = fieldsDatabase.begin(); iter != fieldsDatabase.end(); ++iter)
-    {
-      Glib::ustring field_name = iter->get_name();
-
-       //Look in the result so far:
-       type_vecFields::const_iterator iterFindDatabase = std::find_if(fieldsDocument.begin(), fieldsDocument.end(), predicate_FieldHasName<Field>(field_name));
-
-       //Add it if it is not there:
-       if(iterFindDatabase == fieldsDocument.end() )
-         fieldsDocument.push_back(*iter);
-    }
-
-    return fieldsDocument;
+    field = *iter;
+    return true;
   }
-   
+
+  return false;
 }
 
-Box_DB_Table::type_vecFields Box_DB_Table::get_fields_for_table_from_database(const Glib::ustring& table_name)
-{
-  type_vecFields result;
 
-  // These are documented here:
-  // http://www.gnome-db.org/docs/libgda/libgda-provider-class.html#LIBGDA-PROVIDER-GET-SCHEMA
-  enum GlomGdaDataModelFieldColumns
-  {
-    DATAMODEL_FIELDS_COL_NAME = 0,
-    DATAMODEL_FIELDS_COL_TYPE = 1,
-    DATAMODEL_FIELDS_COL_SIZE = 2,
-    DATAMODEL_FIELDS_COL_SCALE = 3,
-    DATAMODEL_FIELDS_COL_NOTNULL = 4,
-    DATAMODEL_FIELDS_COL_PRIMARYKEY = 5,
-    DATAMODEL_FIELDS_COL_UNIQUEINDEX = 6,
-    DATAMODEL_FIELDS_COL_REFERENCES = 7,
-    DATAMODEL_FIELDS_COL_DEFAULTVALUE = 8
-  };
-
-  Bakery::BusyCursor(*get_app_window());
-
-  sharedptr<SharedConnection> sharedconnection = connect_to_server();
-  if(sharedconnection)
-  {
-    Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
-
-    Gnome::Gda::Parameter param_table_name("name", table_name);
-    Gnome::Gda::ParameterList param_list;
-    param_list.add_parameter(param_table_name);
-
-    Glib::RefPtr<Gnome::Gda::DataModel> data_model_fields = connection->get_schema(Gnome::Gda::CONNECTION_SCHEMA_FIELDS, param_list);
-    if(data_model_fields && (data_model_fields->get_n_columns() == 0))
-    {
-      std::cerr << "Box_DB_Table_Definition::fill_fields(): libgda reported 0 fields for the table." << std::endl;
-    }
-    else if(data_model_fields)
-    {
-      guint row = 0;
-      guint rows_count = data_model_fields->get_n_rows();
-      while(row < rows_count)
-      {
-        Gnome::Gda::FieldAttributes field_info;
-
-        //Get the field name:
-        Gnome::Gda::Value value_name = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_NAME, row);
-        if(value_name.get_value_type() ==  Gnome::Gda::VALUE_TYPE_STRING)
-          field_info.set_name( value_name.get_string() );
-
-        //Get the field type:
-        //This is a string representation of the type, so we need to discover the Gnome::Gda::ValueType for it:
-        Gnome::Gda::Value value_fieldtype = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_TYPE, row);
-        if(value_fieldtype.get_value_type() ==  Gnome::Gda::VALUE_TYPE_STRING)
-        {
-           Glib::ustring schema_type_string = value_fieldtype.get_string();
-           if(!schema_type_string.empty())
-           {
-             ConnectionPool* connection_pool = ConnectionPool::get_instance();
-             const FieldTypes* pFieldTypes = connection_pool->get_field_types();
-             if(pFieldTypes)
-             {
-               Gnome::Gda::ValueType gdatype = pFieldTypes->get_gdavalue_for_schema_type_string(schema_type_string);
-               field_info.set_gdatype(gdatype);
-             }
-           }
-        }
-
-        //Get the default value:
-        /* libgda does not return this correctly yet. TODO: check bug http://bugzilla.gnome.org/show_bug.cgi?id=143576
-        Gnome::Gda::Value value_defaultvalue = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_DEFAULTVALUE, row);
-        if(value_defaultvalue.get_value_type() ==  Gnome::Gda::VALUE_TYPE_STRING)
-          field_info.set_default_value(value_defaultvalue);
-        */
-
-        //Get whether it can be null:
-        Gnome::Gda::Value value_notnull = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_NOTNULL, row);
-        if(value_notnull.get_value_type() ==  Gnome::Gda::VALUE_TYPE_BOOLEAN)
-          field_info.set_allow_null(value_notnull.get_bool());
-
-
-        //Get whether it is a primary key:
-        Gnome::Gda::Value value_primarykey = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_PRIMARYKEY, row);
-        if(value_primarykey.get_value_type() ==  Gnome::Gda::VALUE_TYPE_BOOLEAN)
-          field_info.set_primary_key( value_primarykey.get_bool() );
-
-        //Get whether it is unique
-        Gnome::Gda::Value value_unique = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_UNIQUEINDEX, row);
-        if(value_unique.get_value_type() ==  Gnome::Gda::VALUE_TYPE_BOOLEAN)
-          field_info.set_unique_key( value_unique.get_bool() );
-
-        //Get whether it is autoincrements
-        /* libgda does not provide this yet.
-        Gnome::Gda::Value value_autoincrement = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_AUTOINCREMENT, row);
-        if(value_autoincrement.get_value_type() ==  Gnome::Gda::VALUE_TYPE_BOOLEAN)
-          field_info.set_auto_increment( value_autoincrement.get_bool() );
-        */
-
-        Field field; //TODO: Get glom-specific information from the document?
-        field.set_field_info(field_info);
-        result.push_back(field);
-
-        ++row;
-      }
-    }
-  }
-  
-  return result;
-}
 
 //static:
 Box_DB_Table::type_vecFields Box_DB_Table::get_fields_for_datamodel(const Glib::RefPtr<Gnome::Gda::DataModel>& data_model)
