@@ -47,9 +47,11 @@ public:
   virtual Glib::ustring get_connection_database();
 
   typedef std::vector<Relationship> type_vecRelationships;
-  virtual type_vecRelationships get_relationships(const Glib::ustring& strTableName);
+  virtual type_vecRelationships get_relationships(const Glib::ustring& strTableName) const;
   virtual void set_relationships(const Glib::ustring& strTableName, type_vecRelationships vecRelationships);
 
+  virtual bool get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name, Relationship& relationship) const;
+  
   typedef std::vector<Field> type_vecFields;
   virtual type_vecFields get_table_fields(const Glib::ustring& strTableName) const;
   virtual void set_table_fields(const Glib::ustring& strTableName, type_vecFields vecFields);
@@ -87,6 +89,8 @@ public:
   virtual Glib::ustring get_table_title(const Glib::ustring& table_name) const;
   virtual void set_table_title(const Glib::ustring& table_name, const Glib::ustring value);
 
+  virtual Glib::ustring get_default_table() const;
+  
   virtual AppState::userlevels get_userlevel() const;
 
   /** This is transitory information, not saved to disk.
