@@ -40,10 +40,10 @@ void Box_DB_Table_Definition::init()
   Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_field_definition_edit");
   if(refXml)
     refXml->get_widget_derived("window_field_definition_edit", m_pDialog);
+  add_view(m_pDialog); //Give it access to the document.
 
-  m_colName = 0;
   pack_start(m_AddDel);
-  m_AddDel.add_column(gettext("Name"));
+  m_colName = m_AddDel.add_column(gettext("Name"));
 
   m_colTitle = m_AddDel.add_column(gettext("Title"));
 
@@ -79,6 +79,33 @@ Box_DB_Table_Definition::~Box_DB_Table_Definition()
 {
 }
 
+void Box_DB_Table_Definition::fill_field_row(const Gtk::TreeModel::iterator& iter, const Field& field)
+{
+  m_AddDel.set_value_key(iter, field.get_name());
+  
+  m_AddDel.set_value(iter, m_colName, field.get_name());
+
+  Glib::ustring title = field.get_title();
+  m_AddDel.set_value(iter, m_colTitle, title);
+
+  Gnome::Gda::FieldAttributes fieldinfo = field.get_field_info();
+
+  //Type:
+  Field::glom_field_type fieldType = Field::get_glom_type_for_gda_type(fieldinfo.get_gdatype()); //Could be TYPE_INVALID if the gda type is not one of ours.
+
+  Glib::ustring strType = Field::get_type_name( fieldType );
+  m_AddDel.set_value(iter, m_colType, strType);
+
+  //Unique:
+  bool bUnique = fieldinfo.get_unique_key();
+
+  m_AddDel.set_value(iter, m_colUnique, bUnique);
+
+  //Primary Key:
+  bool bPrimaryKey = fieldinfo.get_primary_key();
+  m_AddDel.set_value(iter, m_colPrimaryKey, bPrimaryKey);
+}
+
 void Box_DB_Table_Definition::fill_from_database()
 {
   Box_DB_Table::fill_from_database();
@@ -95,27 +122,8 @@ void Box_DB_Table_Definition::fill_from_database()
       const Field& field = *iter;
              
       //Name:
-      guint uiRow = m_AddDel.add_item(field.get_name());
-
-      Glib::ustring title = field.get_title();
-      m_AddDel.set_value(uiRow, m_colTitle, title);      
-
-      Gnome::Gda::FieldAttributes fieldinfo = field.get_field_info();
-      
-      //Type:
-      Field::glom_field_type fieldType = Field::get_glom_type_for_gda_type(fieldinfo.get_gdatype()); //Could be TYPE_INVALID if the gda type is not one of ours.
-
-      Glib::ustring strType = Field::get_type_name( fieldType );
-      m_AddDel.set_value(uiRow, m_colType, strType);
-
-      //Unique:
-      bool bUnique = fieldinfo.get_unique_key();
-    
-      m_AddDel.set_value(uiRow, m_colUnique, bUnique);
-
-      //Primary Key:
-      bool bPrimaryKey = fieldinfo.get_primary_key();
-      m_AddDel.set_value(uiRow, m_colPrimaryKey, bPrimaryKey);
+      Gtk::TreeModel::iterator iter= m_AddDel.add_item(field.get_name());
+      fill_field_row(iter, field);
     }
   }
   catch(std::exception& ex)
@@ -126,84 +134,105 @@ void Box_DB_Table_Definition::fill_from_database()
   fill_end();
 }
 
-void Box_DB_Table_Definition::on_AddDel_add(guint row)
+void Box_DB_Table_Definition::on_AddDel_add(const Gtk::TreeModel::iterator& row)
 {
-  Glib::ustring strName = m_AddDel.get_value(row);
-  if(strName.size())
+  Glib::ustring strName = m_AddDel.get_value(row, m_colName);
+  if(!strName.empty())
   {
     bool bTest = Query_execute( "ALTER TABLE " + m_strTableName + " ADD " + strName + " NUMERIC" ); //TODO: Get schema type for Field::TYPE_NUMERIC
-
-    fill_fields();
-
-    fill_from_database();
-
     if(bTest)
     {
-      m_AddDel.select_item(strName);
+      //Show the new field (fill in the other cells):
+      
+      fill_fields();
+
+      //fill_from_database(); //We can not change the structure in a cell renderer signal handler.
+
+      //This must match the SQL statement above:
+      Field field;
+      field.set_name(strName);
+      field.set_glom_type(Field::TYPE_NUMERIC);
+
+      Gnome::Gda::FieldAttributes field_info = field.get_field_info();
+      field_info.set_gdatype( Field::get_gda_type_for_glom_type(Field::TYPE_NUMERIC) );
+      field.set_field_info(field_info);
+      
+      fill_field_row(row, field);
+      
+      //m_AddDel.select_item(row, m_colTitle, true); //Start editing the title
     }
   }
 }
 
-void Box_DB_Table_Definition::on_AddDel_delete(guint rowStart, guint rowEnd)
+void Box_DB_Table_Definition::on_AddDel_delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator&  rowEnd)
 {
-  for(guint row = rowStart; row <= rowEnd; row++)
+  Gtk::TreeModel::iterator iterAfterEnd = rowEnd;
+  if(iterAfterEnd != m_AddDel.get_model()->children().end())
+    ++iterAfterEnd;
+    
+  for(Gtk::TreeModel::iterator iter = rowStart; iter != iterAfterEnd; ++iter)
   {
-    Glib::ustring strName = m_AddDel.get_value(row);
-    if(strName.size())
+    Glib::ustring strName = m_AddDel.get_value_key(iter);
+    if(!strName.empty())
     {
       Query_execute( "ALTER TABLE " + m_strTableName + " DROP COLUMN " + strName );
     }
   }
 
+  fill_fields();
   fill_from_database();
 }
 
-void Box_DB_Table_Definition::on_AddDel_changed(guint row, guint /* col */)
+void Box_DB_Table_Definition::on_AddDel_changed(const Gtk::TreeModel::iterator& row, guint /* col */)
 {
   //Get old field definition:
   Document_Glom* pDoc = static_cast<Document_Glom*>(get_document());
   if(pDoc)
   {
-    //Glom-specific stuff:
-    const Glib::ustring& strFieldNameBeingEdited = m_Fields[row].get_name();
+    //Glom-specific stuff: //TODO_portiter
+    const Glib::ustring strFieldNameBeingEdited = m_AddDel.get_value_key(row);
+    
     pDoc->get_field(m_strTableName, strFieldNameBeingEdited, m_Field_BeingEdited);
 
     //Get DB field info: (TODO: This might be unnecessary).
-    m_Field_BeingEdited = m_Fields[row];
-
-
-    //Get new field definition:
-    Field fieldNew = get_field_definition(row);
-        
-    //Change it:
-    if(m_Field_BeingEdited != fieldNew) //If it has really changed.
+    type_vecFields::const_iterator iterFind = std::find_if( m_Fields.begin(), m_Fields.end(), predicate_FieldHasName<Field>(strFieldNameBeingEdited) );
+    if(iterFind != m_Fields.end()) //If it was found:
     {
-      //If we are changing a non-glom type:
-     //Refuse to edit field definitions that were not created by glom:
-     if(Field::get_glom_type_for_gda_type( m_Field_BeingEdited.get_field_info().get_gdatype() )  == Field::TYPE_INVALID)
-     {
-       Gtk::MessageDialog dialog(gettext("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
-       dialog.set_transient_for(*get_app_window());
-       dialog.run();
-     }
-     else
-       change_definition(m_Field_BeingEdited, fieldNew);
+      m_Field_BeingEdited = *iterFind;
+
+      //Get new field definition:
+      Field fieldNew = get_field_definition(row);
+       
+      //Change it:
+      if(m_Field_BeingEdited != fieldNew) //If it has really changed.
+      {
+        //If we are changing a non-glom type:
+       //Refuse to edit field definitions that were not created by glom:
+       if(Field::get_glom_type_for_gda_type( m_Field_BeingEdited.get_field_info().get_gdatype() )  == Field::TYPE_INVALID)
+       {
+         Gtk::MessageDialog dialog(gettext("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
+         dialog.set_transient_for(*get_app_window());
+         dialog.run();
+       }
+       else
+         change_definition(m_Field_BeingEdited, fieldNew);
+      }
     }
   }
 }
 
-void Box_DB_Table_Definition::on_AddDel_edit(guint row)
+void Box_DB_Table_Definition::on_AddDel_edit(const Gtk::TreeModel::iterator& row)
 {
   m_Field_BeingEdited = get_field_definition(row);
   
-  m_pDialog->set_field(m_Field_BeingEdited); //By-value.
+  m_pDialog->set_field(m_Field_BeingEdited, m_strTableName);
  
   //m_pDialog->set_modified(false); //Disable [Apply] at start.
   
   m_pDialog->show_all();
 }
 
-Field Box_DB_Table_Definition::get_field_definition(guint row)
+Field Box_DB_Table_Definition::get_field_definition(const Gtk::TreeModel::iterator& row)
 {
   Field fieldResult;
 
@@ -213,7 +242,7 @@ Field Box_DB_Table_Definition::get_field_definition(guint row)
   Document_Glom* pDoc = static_cast<Document_Glom*>(get_document());
   if(pDoc)
   {
-    const Glib::ustring& strFieldNameBeforeEdit = m_Fields[row].get_name();
+    const Glib::ustring& strFieldNameBeforeEdit = m_AddDel.get_value(row, 0);
 
     Document_Glom::type_vecFields vecFields= pDoc->get_table_fields(m_strTableName);
     Document_Glom::type_vecFields::iterator iterFind = std::find_if( vecFields.begin(), vecFields.end(), predicate_FieldHasName<Field>(strFieldNameBeforeEdit) );
@@ -232,8 +261,8 @@ Field Box_DB_Table_Definition::get_field_definition(guint row)
 
   //Start with original definitions, so that we preserve things like UNSIGNED.
   //TODO maybe use document's fieldinfo instead of m_Fields.
-  Field field = m_Fields[row];
-  Gnome::Gda::FieldAttributes fieldInfo = field.get_field_info();
+  Field field_temp = m_Fields[row];
+  Gnome::Gda::FieldAttributes fieldInfo = field_temp.get_field_info();
 
   //Name:
   Glib::ustring strName = m_AddDel.get_value(row, m_colName);
@@ -241,7 +270,7 @@ Field Box_DB_Table_Definition::get_field_definition(guint row)
 
   //Title:
   Glib::ustring title = m_AddDel.get_value(row, m_colTitle);
-  field.set_title(title);
+  fieldResult.set_title(title);
 
   //Type:
   const Glib::ustring& strType = m_AddDel.get_value(row, m_colType);
@@ -261,7 +290,7 @@ Field Box_DB_Table_Definition::get_field_definition(guint row)
 
   //Put it together:
   fieldResult.set_field_info(fieldInfo);
-
+      
   return fieldResult;
 }
 
@@ -284,6 +313,9 @@ void Box_DB_Table_Definition::on_Properties_apply()
       change_definition(m_Field_BeingEdited, field_New);
       m_Field_BeingEdited = field_New;
     }
+
+    //Update the list:
+    fill_from_database();
   }
 
   m_pDialog->hide();
@@ -316,29 +348,7 @@ void Box_DB_Table_Definition::change_definition(const Field& fieldOld, Field fie
     }
   }
   */
-
-  
-  /* Old MySQL stuff:
-  if(fieldInfoOld.get_primary_key())
-  {
-    //If we are modifying a primary key then we first need to stop it from being the primary key.
-    //Otherwise mysql complains that there is a conflict. Silly mysql.
-
-    Gnome::Gda::FieldAttributes fieldInfoNonPrimary = fieldInfoOld;
-    fieldInfoNonPrimary.set_primary_key(false);
-    fieldInfoNonPrimary.set_auto_increment(false);
-    Query_execute( "ALTER TABLE " + m_strTableName + " CHANGE " + strFieldNameOld + " " + fieldInfoNonPrimary.get_name() + " " + get_field_definition_for_sql(fieldInfoNonPrimary) );
-
-    Query_execute( "ALTER TABLE " + m_strTableName + " DROP PRIMARY KEY");
-  }
-  else if(fieldInfoOld.get_unique_key() && fieldInfo.get_unique_key())
-  {
-    //It doesn't seem possible to turn off UNIQUE.
-    //And specifying it a second time gives an error.
-
-    fieldInfo.set_unique_key(false);
-  }
-  */
+ 
 
    //MySQL does this all with ALTER_TABLE, with "CHANGE" followed by the same details used with "CREATE TABLE",
    //MySQL also makes it easier to change the type.
@@ -378,10 +388,10 @@ void Box_DB_Table_Definition::change_definition(const Field& fieldOld, Field fie
   //Update UI:
 
   fill_fields();
-  fill_from_database();
+  //fill_from_database(); //We should not change the database definition in a cell renderer signal handler.
 
   //Select the same field again:
-  m_AddDel.select_item(field.get_name());
+  m_AddDel.select_item(field.get_name(), m_colName, false);
 }
 
 void Box_DB_Table_Definition::fill_fields()
@@ -389,8 +399,6 @@ void Box_DB_Table_Definition::fill_fields()
   m_Fields = get_fields_for_table(m_strTableName);
 }
 
-
-
 void  Box_DB_Table_Definition::postgres_add_column(const Field& field, bool not_extras)
 {
   bool bTest = Query_execute(  "ALTER TABLE " + m_strTableName + " ADD " + field.get_name() + " " +  field.get_sql_type() );
