@@ -63,7 +63,7 @@ AddDelColumnInfo& AddDelColumnInfo::operator=(const AddDelColumnInfo& src)
 }
 
 AddDel::AddDel()
-: m_bHasRowTitles(false),
+:  m_col_key(0),
   m_pColumnHeaderPopup(0),
   m_allow_column_chooser(false),
   m_auto_add(true),
@@ -106,6 +106,8 @@ AddDel::AddDel()
   set_prevent_user_signals(false);
   set_ignore_treeview_signals(false);
 
+  remove_all_columns(); //set up the default columns.
+  
   show_all_children();
 }
 
@@ -122,41 +124,30 @@ AddDel::on_MenuPopup_activate_Edit()
     Gtk::TreeModel::iterator iter = refSelection->get_selected();
     if(iter)
     {
-      int row = row_number_from_iterator(iter);
+      //Discover whether it's the last (empty) row:
+      if(get_is_placeholder_row(iter))
+      {        
+        //This is a new entry:
+        signal_user_added()(iter);
 
-      Glib::ustring strValue_Old = get_value(row);
+        /*
+        bool bRowAdded = true;
 
-      finish_editing();
-
-      Glib::ustring strValue = get_value(row);
-
-      if(strValue.size())
-      {
-        bool bNewEntryPos = (row == (int)m_refListStore->children().size() - 1);
-        if(strValue_Old.size() && bNewEntryPos)
-        {
-          //This is a new entry:
-          signal_user_added()(row);
-
-          bool bRowAdded = true;
-
-          //The rows might be re-ordered:
-          guint rowAdded = row;
-          Glib::ustring strValue_Added =  get_value(row);
-          if(strValue_Added != strValue)
-            bRowAdded = get_row_number(strValue, rowAdded);
-
-          if(bRowAdded)
-            signal_user_requested_edit()(rowAdded);
-        }
-        else
-        {
-          //Value changed:
-          signal_user_requested_edit()(row);
-        }
+        //The rows might be re-ordered:
+        Gtk::TreeModel::iterator rowAdded = iter;
+        Glib::ustring strValue_Added =  get_value_key(iter);
+        if(strValue_Added != strValue)
+          rowAdded = get_row(strValue);
 
+        if(bRowAdded)
+          signal_user_requested_edit()(rowAdded);
+        */
+      }
+      else
+      {
+        //Value changed:
+        signal_user_requested_edit()(iter);
       }
-
     }
 
   }
@@ -164,10 +155,10 @@ AddDel::on_MenuPopup_activate_Edit()
 
 void AddDel::on_MenuPopup_activate_ChooseColumns()
 {
-	GtkDialog* pDialog = GTK_DIALOG( egg_column_chooser_dialog_new(m_TreeView.gobj()) );
-	gtk_widget_show_all( GTK_WIDGET(pDialog) );
-	gtk_dialog_run(pDialog); //It will delete itself. The C API is funny like that.
-	pDialog = 0;
+  GtkDialog* pDialog = GTK_DIALOG( egg_column_chooser_dialog_new(m_TreeView.gobj()) );
+  gtk_widget_show_all( GTK_WIDGET(pDialog) );
+  gtk_dialog_run(pDialog); //It will delete itself. The C API is funny like that.
+  pDialog = 0;
 }
 
 void AddDel::on_MenuPopup_activate_Delete()
@@ -181,8 +172,7 @@ void AddDel::on_MenuPopup_activate_Delete()
     if(iter)
     {
       //TODO: We can't handle multiple-selections yet.
-      int rowStart = row_number_from_iterator(iter);
-      signal_user_requested_delete().emit(rowStart, rowStart);
+      signal_user_requested_delete().emit(iter, iter);
     }
   }
 }
@@ -206,9 +196,9 @@ void AddDel::setup_menu()
   
   if(m_allow_column_chooser)
   {
-	  Gtk::Menu_Helpers::MenuElem menuitem_ChooseColumns(gettext("Choose columns"));
-	  m_MenuPopup.items().push_back(menuitem_ChooseColumns);
-	  m_MenuPopup.items().back().signal_activate().connect(sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_ChooseColumns));
+    Gtk::Menu_Helpers::MenuElem menuitem_ChooseColumns(gettext("Choose columns"));
+    m_MenuPopup.items().push_back(menuitem_ChooseColumns);
+    m_MenuPopup.items().back().signal_activate().connect(sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_ChooseColumns));
   }
 
 }
@@ -234,66 +224,49 @@ bool AddDel::on_button_press_event_Popup(GdkEventButton *event)
   return true;
 }
 
-guint AddDel::add_item()
+Gtk::TreeModel::iterator AddDel::get_item_placeholder()
 {
-  return add_item("");
-}
-
-int AddDel::get_first_column() const
-{
-  return (m_bHasRowTitles ? 1 : 0);
-}
-
-guint AddDel::add_item(const Glib::ustring& strKey)
-{
-  if(!m_refListStore)
-    return 0;
-
-  //The deactivate signals seems to cause the current cell to revert to it's previsous value.
-  //InnerIgnore innerIgnore(this); //see comments for InnerIgnore class.
-
-  //Add a new line:
-  //The extra blank is not used if the user may not add items:
-
-  if(get_allow_user_actions())
+  //Get the existing placeholder row, or add one if necessary:
+  Gtk::TreeModel::iterator iter = get_last_row();
+  if( get_is_placeholder_row(iter) )
   {
-    if(get_blank_is_used())
-    {
-      add_blank();
-    }
+    return iter;
   }
   else
   {
-      Gtk::TreeModel::iterator storeIter = m_refListStore->append();  //Just add it:
-      //m_refListStore->set_value(jstoreIter, 0, "");
+    return add_item_placeholder();
   }
-
-  guint uiRow = m_refListStore->children().size() - 1;  //There must be at least 1 row now.
-  Gtk::TreeModel::Row treerow = m_refListStore->children()[uiRow];
-  if(treerow)
+}
+  
+Gtk::TreeModel::iterator AddDel::add_item_placeholder()
+{
+  Gtk::TreeModel::iterator iter = m_refListStore->append();
+  if(iter)
   {
-    treerow.set_value(get_first_column(), strKey);
+    iter->set_value(m_col_key, Glib::ustring("")); //Remove temporary key value.
+    iter->set_value(m_col_placeholder, true); //This will be unset when set_value() is used to put real data in this column.
   }
 
-  bool bExtraBlank = false;
-  if(strKey.size())
+  return iter;
+}
+
+Gtk::TreeModel::iterator AddDel::add_item(const Glib::ustring& strKey)
+{
+  Gtk::TreeModel::iterator result = get_next_available_row_with_add_if_necessary();
+
+  if(result)
   {
-    //There should always be 1 extra blank.
-    //Unless this AddDel does not allow the user to add new items:
-    if(get_allow_user_actions())
-      bExtraBlank = add_blank();
+    Gtk::TreeModel::Row treerow = *result;
+    if(treerow)
+    {
+      result->set_value(m_col_key, strKey);
+      result->set_value(m_col_placeholder, false);
+    }
   }
 
-  //select_item(uiRow);
-  //Don't use select_item() because we need to return false from on_sheet_deactivate().
+  add_blank(); //if necessary
 
-
-  //if(uiRow < m_Sheet.get_rows_count())
-  //  m_Sheet.set_active_cell(uiRow, 0);
-
-
-
-  return uiRow;
+  return result;
 }
 
 void AddDel::remove_all()
@@ -311,23 +284,24 @@ void AddDel::remove_all()
   }
 }
 
-Gnome::Gda::Value AddDel::get_value_as_value(guint row, guint col)
+
+Gnome::Gda::Value AddDel::get_value_as_value(const Gtk::TreeModel::iterator& iter, guint col)
 {
   bool success = false;
-  return GlomConversions::parse_value(m_ColumnTypes[col].m_field_type, get_value(row, col), success );
+  return GlomConversions::parse_value(m_ColumnTypes[col].m_field_type, get_value(iter, col), success );
 }
 
-Glib::ustring AddDel::get_value(guint row, guint col)
+Glib::ustring AddDel::get_value(const Gtk::TreeModel::iterator& iter, guint col)
 {
   Glib::ustring value;
 
   if(m_refListStore)
   {
-    Gtk::TreeModel::Row treerow = m_refListStore->children()[row];
+    Gtk::TreeModel::Row treerow = *iter;
 
     if(treerow)
     {
-      const guint col_real = get_first_column() + col;
+      const guint col_real = col;
       //Get different types of data, depending on the column:
       if(m_ColumnTypes[col_real].m_style == AddDelColumnInfo::STYLE_Boolean)
       {
@@ -347,14 +321,26 @@ Glib::ustring AddDel::get_value(guint row, guint col)
   //TODO: This is a hack. We need to deal with NULL explicitly:
   if(value == "NULL")
     value = "";
-    
+
   return value;
 }
 
-bool AddDel::get_value_as_bool(guint row, guint col)
+bool AddDel::get_value_as_bool(const Gtk::TreeModel::iterator& iter, guint col)
 {
-  Glib::ustring strValue = get_value(row, col);
-  return (strValue == "true");
+//TODO: I doubt that this works. It should really get the value from the treeview as a bool. murrayc
+  Glib::ustring strValue = get_value(iter, col);
+  return (strValue == "true");   
+}
+
+Glib::ustring AddDel::get_value_key_selected()
+{
+  return get_value_selected(m_col_key);
+}
+
+Gnome::Gda::Value AddDel::get_value_key_selected_as_value()
+{
+  bool parsed = false;
+  return GlomConversions::parse_value(m_key_field.get_glom_type(), get_value_key_selected(), parsed);
 }
 
 Glib::ustring AddDel::get_value_selected(guint col)
@@ -363,67 +349,61 @@ Glib::ustring AddDel::get_value_selected(guint col)
   return strValue;
 }
 
-guint AddDel::get_item_selected()
+Gtk::TreeModel::iterator AddDel::get_item_selected()
 {
   Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
   if(refTreeSelection)
   {
-     Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
-     if(iter)
-       return row_number_from_iterator(iter);
+     return refTreeSelection->get_selected();
   }
 
-  return 0;
+  return m_refListStore->children().end();
 }
 
-bool AddDel::get_row_number(const Glib::ustring& strItemText, guint& row)
+Gtk::TreeModel::iterator AddDel::get_row(const Glib::ustring& key)
 {
-  row = 0;
-
-  guint rowCount = get_count();
-
-  //Check each item and compare text:
-  for(guint i = 0; i < rowCount; i++)
+  for(Gtk::TreeModel::iterator iter = m_refListStore->children().begin(); iter != m_refListStore->children().end(); ++iter)
   {
-    const Glib::ustring& strTemp = get_value(i, get_first_column());
-    if(strTemp == strItemText)
+    const Glib::ustring strTemp = get_value(iter, m_col_key);
+    if(strTemp == key)
     {
-      row = i;
-      return true; //found:
+      return iter;
     }
   }
 
-  return false; //not found.
+  return  m_refListStore->children().end();
 }
 
-bool AddDel::select_item(guint row, bool start_editing)
-{
-  return select_item(row, get_first_column(), start_editing);
-}
-
-bool AddDel::select_item(guint row, guint column, bool start_editing)
+bool AddDel::select_item(const Gtk::TreeModel::iterator& iter, guint column, bool start_editing)
 {
   if(!m_refListStore)
     return false;
-
+   
   InnerIgnore innerIgnore(this); //see comments for InnerIgnore class
 
   bool bResult = false;
 
-  if(row < m_refListStore->children().size())
+  if(iter)
   {
-    Gtk::TreeModel::Row treerow = m_refListStore->children()[row];
+    Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
+    if(refTreeSelection)
     {
-      Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
-      if(refTreeSelection)
+      refTreeSelection->select(iter);
+          
+      Gtk::TreeModel::Path path = m_refListStore->get_path(iter);
+
+      guint view_column_index = 0;
+      bool test = get_view_column_index(column, view_column_index);
+      if(test)
       {
-        refTreeSelection->select(treerow);
-
-        Gtk::TreeModel::Path path = m_refListStore->get_path(treerow);
-        Gtk::TreeView::Column* pColumn = m_TreeView.get_column(column);
-
-        m_TreeView.set_cursor(path, *pColumn, start_editing);
+        Gtk::TreeView::Column* pColumn = m_TreeView.get_column(view_column_index);
+        if(pColumn)
+          m_TreeView.set_cursor(path, *pColumn, start_editing);
+        else
+           g_warning("AddDel::select_item:TreeViewColumn not found.");
       }
+      else
+           g_warning("AddDel::select_item:TreeViewColumn index not found.");
     }
 
     bResult = true;
@@ -432,16 +412,15 @@ bool AddDel::select_item(guint row, guint column, bool start_editing)
   return bResult;
 }
 
-bool AddDel::select_item(const Glib::ustring& strItemText)
+bool AddDel::select_item(const Glib::ustring& strItemText, guint column, bool start_editing)
 {
-  guint row = 0;
-  bool bTest = get_row_number(strItemText, row);
-  if(bTest)
+  Gtk::TreeModel::iterator iter = get_row(strItemText);
+  if(iter)
   {
-    select_item(row);
+    return select_item(iter, column, start_editing);
   }
 
-  return bTest;
+  return false; //failed
 }
 
 guint AddDel::get_count() const
@@ -452,24 +431,17 @@ guint AddDel::get_count() const
   guint iCount = m_refListStore->children().size();
 
   //Take account of the extra blank for new entries:
-  if(iCount > 0) //Gtk::Extra sheet always has at least 1 row.
+  if(get_allow_user_actions()) //If it has the extra row.
   {
-    if(get_allow_user_actions()) //If it has the extra row.
-    {
-      //Look at the last row:
-      if(!get_blank_is_used())
-           iCount--;
-    }
+    if(get_is_placeholder_row(get_last_row()))
+      --iCount;
   }
 
   return iCount;
 }
 
-bool AddDel::add_blank()
+void AddDel::add_blank()
 {
-  if(!m_refListStore)
-    return false;
-
   bool bPreventUserSignals = get_prevent_user_signals();
   set_prevent_user_signals(true);
 
@@ -477,57 +449,23 @@ bool AddDel::add_blank()
 
   if(get_allow_user_actions()) //The extra blank line is only used if the user may add items:
   {
-    guint rowsCount = m_refListStore->children().size();
-    if(rowsCount)
+    Gtk::TreeModel::iterator iter = get_last_row();        
+    if(get_is_placeholder_row(iter))
     {
-        //Look at the last row:
-      Glib::ustring strValue = get_value(rowsCount-1, 0);
-      if(strValue.size() == 0)
-      {
         bAddNewBlank  = false; //One already exists.
-      }
-      else
-      {
-        bAddNewBlank = true; // The last line isn't blank. Add one.
-      }
     }
     else
     {
-      bAddNewBlank = true; //Add first row.
+      bAddNewBlank = true; // The last line isn't a placeholder. Add one.
     }
   }
 
   if(bAddNewBlank)
-    m_refListStore->append();; //Add the next blank for the next user add.
+    add_item_placeholder();
 
   set_prevent_user_signals(bPreventUserSignals);
-
-  return bAddNewBlank;
 }
 
-bool AddDel::get_blank_is_used() const
-{
-  if(!m_refListStore)
-   return false;
-
-  guint iCount = m_refListStore->children().size();
-
-  if(iCount == 0)
-  {
-    return true; //Shouldn't happen. Force creation of new blank.
-  }
-
-  Glib::ustring strValue;
-
-  //Look at the last row:
-  Gtk::TreeModel::Row row = m_refListStore->children()[iCount-1];
-  if(row)
-  {
-     row.get_value(get_first_column(), strValue);
-  }
-
-  return (strValue.size() > 0);
-}
 
 guint AddDel::get_columns_count() const
 {
@@ -615,18 +553,19 @@ void AddDel::construct_specified_columns()
   m_TreeView.remove_all_columns();
 
   //Add new View Colums:
-  int iColumn = 0;
+  int model_column_index = 0;
+  int view_column_index = 0;
   for(type_vecModelColumns::iterator iter = vecModelColumns.begin(); iter != vecModelColumns.end(); ++iter)
   {
     type_vecModelColumns::value_type& pModelColumn = *iter;
 
-    if(m_ColumnTypes[iColumn].m_visible)
+    if(m_ColumnTypes[model_column_index].m_visible)
     {
-      const Glib::ustring column_name = m_ColumnTypes[iColumn].m_name;
-      const Glib::ustring column_id = m_ColumnTypes[iColumn].m_id;
+      const Glib::ustring column_name = m_ColumnTypes[model_column_index].m_name;
+      const Glib::ustring column_id = m_ColumnTypes[model_column_index].m_id;
 
       int cols_count = 0;
-      switch(m_ColumnTypes[iColumn].m_style)
+      switch(m_ColumnTypes[model_column_index].m_style)
       {
         case(AddDelColumnInfo::STYLE_Choices):
         {
@@ -634,7 +573,7 @@ void AddDel::construct_specified_columns()
           CellRendererList* pCellRenderer = Gtk::manage( new CellRendererList() );
 
           //Add the choices:
-          const type_vecStrings vecStrings = m_ColumnTypes[iColumn].m_choices;
+          const type_vecStrings vecStrings = m_ColumnTypes[model_column_index].m_choices;
           for(type_vecStrings::const_iterator iter = vecStrings.begin(); iter != vecStrings.end(); ++iter)
           {
             pCellRenderer->append_list_item(*iter);
@@ -672,13 +611,11 @@ void AddDel::construct_specified_columns()
 
           break;
         }
-      }
+      } //switch
 
-      const int model_column_index = cols_count - 1;
-
-      if(m_ColumnTypes[iColumn].m_editable)
+      if(m_ColumnTypes[model_column_index].m_editable)
       {
-        Gtk::CellRendererText* pCellRenderer = dynamic_cast<Gtk::CellRendererText*>(m_TreeView.get_column_cell_renderer(model_column_index));
+        Gtk::CellRendererText* pCellRenderer = dynamic_cast<Gtk::CellRendererText*>(m_TreeView.get_column_cell_renderer(view_column_index));
         if(pCellRenderer)
         {
           //Connect a signal handler:
@@ -694,7 +631,7 @@ void AddDel::construct_specified_columns()
         }
         else
         {
-           Gtk::CellRendererToggle* pCellRenderer = dynamic_cast<Gtk::CellRendererToggle*>(m_TreeView.get_column_cell_renderer(model_column_index));
+           Gtk::CellRendererToggle* pCellRenderer = dynamic_cast<Gtk::CellRendererToggle*>(m_TreeView.get_column_cell_renderer(view_column_index));
            if(pCellRenderer)
            {
              pCellRenderer->property_activatable() = true;
@@ -706,16 +643,18 @@ void AddDel::construct_specified_columns()
         }
       }
 
+    ++view_column_index;
+    
+    } //is visible
 
-      iColumn++;
-    }
+    ++model_column_index;
   }
   
   //Delete the vector's items:
   for(type_vecModelColumns::iterator iter = vecModelColumns.begin(); iter != vecModelColumns.end(); ++iter)
   {
      Gtk::TreeModelColumnBase* pModelColumn = *iter;
-     switch(m_ColumnTypes[iColumn].m_style)
+     switch(m_ColumnTypes[model_column_index].m_style)
      {
        //Cast it to the derived type, so we can delete it properly.
        //This is necessary because TreeModelColumnBase's destructor is not virtual.
@@ -739,47 +678,26 @@ void AddDel::construct_specified_columns()
   m_TreeView.columns_autosize();
 }
 
-
-void AddDel::set_item_title(guint row, const Glib::ustring& strValue)
-{
-  //Set first column text, if it's used for titles:
-  if(m_bHasRowTitles)
-  {
-    InnerIgnore innerIgnore(this);
-
-    if(!m_refListStore)
-      g_warning("AddDel::set_item_title: No model.");
-    else
-    {
-      Gtk::TreeModel::Row treerow = m_refListStore->children()[row];
-      if(treerow)
-        treerow.set_value(0, strValue);
-
-      //Add extra blank if necessary:
-      add_blank();
-    }
-  }
-}
-
-void AddDel::set_value(guint row, guint col, const Gnome::Gda::Value& value)
+void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, const Gnome::Gda::Value& value)
 {
   //Different model columns have different types of data:
   switch(m_ColumnTypes[col].m_style)
   {
     case(AddDelColumnInfo::STYLE_Boolean):
     {
-      set_value(row, col, value.get_bool());
+      g_warning("AddDel::set_value(): boolean column being set as bool.");
+      set_value(iter, col, value.get_bool());
       break;
     }
     default:
     {
-      set_value( row, col, GlomConversions::get_text_for_gda_value(m_ColumnTypes[col].m_field_type, value) );
+      set_value( iter, col, GlomConversions::get_text_for_gda_value(m_ColumnTypes[col].m_field_type, value) );
       break;
     }
   }
 }
 
-void AddDel::set_value(guint row, guint col, const Glib::ustring& strValue)
+void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, const Glib::ustring& strValue)
 {
   InnerIgnore innerIgnore(this);
 
@@ -787,25 +705,34 @@ void AddDel::set_value(guint row, guint col, const Glib::ustring& strValue)
     g_warning("AddDel::set_value: No model.");
   else
   {
-    const guint col_real = get_first_column() + col;
-    Gtk::TreeModel::Row treerow = m_refListStore->children()[row];
+    Gtk::TreeModel::Row treerow = *iter;
     if(treerow)
     {
       //Different model columns have different types of data:
-      switch(m_ColumnTypes[col_real].m_style)
+      switch(m_ColumnTypes[col].m_style)
       {
         case(AddDelColumnInfo::STYLE_Boolean):
         {
           bool bValue = (strValue == "true");
-          treerow.set_value(col_real, bValue);
+          treerow.set_value(col, bValue);
           break;
         }
         default:
         {
-          treerow.set_value(col_real, strValue);
+          treerow.set_value(col, strValue);
           break;
         }
       }
+
+      //Mark this row as not a placeholder because it has real data now.
+      if( (col != m_col_key) && (col != m_col_placeholder) )
+      {
+        if(!strValue.empty())
+        {
+           //treerow.set_value(m_col_key, Glib::ustring("placeholder debug value setted"));
+           //treerow.set_value(m_col_placeholder, false);
+        }
+      }
     }
 
     //Add extra blank if necessary:
@@ -813,25 +740,14 @@ void AddDel::set_value(guint row, guint col, const Glib::ustring& strValue)
   }
 }
 
-void AddDel::set_value(guint row, guint col, unsigned long ulValue)
+void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, unsigned long ulValue)
 {
   gchar pchValue[10] = {0};
   sprintf(pchValue, "%d", (guint)ulValue);
-  set_value(row, col, Glib::ustring(pchValue));
-
-  /*
-  {
-  gchar* pchValue = g_strdup_printf("%d", ulValue);
-  if(pchValue)
-  {
-    set_value(row, col, Glib::ustring(pchValue)
-    g_free(pchValue);
-  }
-  }
-  */
+  set_value(iter, col, Glib::ustring(pchValue));
 }
 
-void AddDel::set_value(guint row, guint col, bool bVal)
+void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, bool bVal)
 {
   InnerIgnore innerIgnore(this);
 
@@ -839,23 +755,21 @@ void AddDel::set_value(guint row, guint col, bool bVal)
     g_warning("AddDel::set_value: No model.");
   else
   {
-    const guint col_real = get_first_column() + col;
-
-    Gtk::TreeModel::Row treerow = m_refListStore->children()[row];
+    Gtk::TreeModel::Row treerow = *iter;
     if(treerow)
     {
       //Different model columns have different types of data:
-      switch(m_ColumnTypes[col_real].m_style)
+      switch(m_ColumnTypes[col].m_style)
       {
         case(AddDelColumnInfo::STYLE_Boolean):
         {
-          treerow.set_value(col_real, bVal);
+          treerow.set_value(col, bVal);
           break;
         }
         default:
         {
           Glib::ustring strValue = (bVal ? "true" : "false");
-          treerow.set_value(col_real, strValue);
+          treerow.set_value(col, strValue);
           break;
         }
       }
@@ -869,6 +783,11 @@ void AddDel::set_value(guint row, guint col, bool bVal)
 void AddDel::remove_all_columns()
 {
   m_ColumnTypes.clear();
+
+  //Add the hidden key.ID columns
+  //Make these visible (with true) if you want to debug problems.
+  m_col_key = add_column("Glom Hidden Key", AddDelColumnInfo::STYLE_Text, false /* not editable */, false /* not visible */);
+  m_col_placeholder = add_column("Glom Hidden Placeholder", AddDelColumnInfo::STYLE_Boolean, true /* not editable */, false /* not visible */);
 }
 
 guint AddDel::add_column(const AddDelColumnInfo& column_info)
@@ -902,6 +821,10 @@ guint AddDel::add_column(const Field& field, AddDelColumnInfo::enumStyles style,
   column_info.m_editable = editable;
   column_info.m_visible = visible;
 
+  //Make it non-editable if it is auto-generated:
+  if(field.get_field_info().get_auto_increment())
+    column_info.m_editable = false;
+
   return add_column(column_info);
 }
   
@@ -983,14 +906,6 @@ Glib::ustring AddDel::get_select_text() const
   return m_strSelectText;
 }
 
-void AddDel::set_show_row_titles(bool bVal)
-{
-  m_bHasRowTitles = bVal;
-
-  //TODO_port
-  //add_column("", STYLE_Text, false /* not editable */);
-}
-
 void AddDel::set_show_column_titles(bool bVal)
 {
   m_TreeView.set_headers_visible(bVal);
@@ -1045,14 +960,11 @@ void AddDel::reactivate()
 //  m_Sheet.set_active_cell(row, col);
 }
 */
-void AddDel::remove_item(guint row)
+
+void AddDel::remove_item(const Gtk::TreeModel::iterator& iter)
 {
-  if(row < get_count())
-  {
-    Gtk::TreeModel::Row treerow = m_refListStore->children()[row];
-    if(treerow)
-      m_refListStore->erase(treerow);
-  }
+  if(iter)
+    m_refListStore->erase(iter);
 }
 
 AddDel::InnerIgnore::InnerIgnore(AddDel* pOuter)
@@ -1082,7 +994,7 @@ AddDel::InnerIgnore::~InnerIgnore()
   m_pOuter = false;
 }
 
-Glib::ustring AddDel::treeview_get_key(guint row)
+Glib::ustring AddDel::treeview_get_key(const Gtk::TreeModel::iterator& row)
 {
   Glib::ustring value;
 
@@ -1091,7 +1003,7 @@ Glib::ustring AddDel::treeview_get_key(guint row)
     Gtk::TreeModel::Row treerow = m_refListStore->children()[row];
 
     if(treerow)
-      treerow.get_value(get_first_column(), value);
+      treerow.get_value(m_col_key, value);
   }
 
   return value;
@@ -1114,8 +1026,7 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
     //Store the user's new value in the model:
     row.set_value(model_column_index, value_new);
 
-    int row_number = row_number_from_iterator(iter);
-
+    //TODO: Did it really change?
     
     //Is this an add or a change?:
 
@@ -1127,12 +1038,12 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
     {
       if(get_allow_user_actions()) //If add is possible:
       {
-        if( (model_column_index == (int)get_first_column() ) && (row_number == (iCount - 1)) ) //If it's the last row:
+        if( get_is_placeholder_row(iter) ) //If it's the last row:
         {
           //We will ignore editing of bool values in the blank row. It seems like a bad way to start a new record.
           //New item in the blank row:
           /*
-          Glib::ustring strValue = get_value(row_number);
+          Glib::ustring strValue = get_value(row);
           if(strValue.size())
           {
             bool bPreventUserSignals = get_prevent_user_signals();
@@ -1158,13 +1069,13 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
          
       //Signal that a new key was added:
       //We will ignore editing of bool values in the blank row. It seems like a bad way to start a new record.
-      //m_signal_user_added.emit(row_number);
+      //m_signal_user_added.emit(row);
     }
     else if(bIsChange)
     {
       //Existing item changed:
 
-      m_signal_user_changed.emit(row_number, model_column_index);
+      m_signal_user_changed.emit(row, model_column_index);
     }
     
   }
@@ -1176,7 +1087,7 @@ void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Gli
 
   //Get the row from the path:
   Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
-  if(iter)
+  if(iter != get_model()->children().end())
   {
     Gtk::TreeModel::Row row = *iter;
 
@@ -1186,43 +1097,38 @@ void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Gli
     //Store the user's new text in the model:
     row.set_value(model_column_index, new_text);
 
-    int row_number = row_number_from_iterator(iter);
+    if(strTextOld == new_text)
+      return; //This is not actually an edit.
 
     //Is it an add or a change?:
-
     bool bIsAdd = false;
     bool bIsChange = false;
 
-    int iCount = m_refListStore->children().size();
-    if(iCount)
+    if(get_allow_user_actions()) //If add is possible:
     {
-      if(get_allow_user_actions()) //If add is possible:
+      if(get_is_placeholder_row(iter))
       {
-        if( (model_column_index == (int)get_first_column() ) && (row_number == (iCount - 1)) ) //If it's the last row:
-        {
-          //New item in the blank row:
-          Glib::ustring strValue = get_value(row_number);
-          if(strValue.size())
-           {
-             bool bPreventUserSignals = get_prevent_user_signals();
-             set_prevent_user_signals(true); //Stops extra signal_user_changed.
-             add_item(); //Add the next blank for the next user add.
-             set_prevent_user_signals(bPreventUserSignals);
+        bool bPreventUserSignals = get_prevent_user_signals();
+        set_prevent_user_signals(true); //Stops extra signal_user_changed.
+        
+        //Mark this row as no longer a placeholder, because it has data now. The client code must set an actual key for this in the signal_user_added() or m_signal_user_changed signal handlers.
+        set_value_key(iter, "glom_unknown");
+        
+        add_item_placeholder(); //Add the next blank for the next user add.
+        set_prevent_user_signals(bPreventUserSignals);
 
-             bIsAdd = true; //Signal that a new key was added.
-          }
-        }
+        bIsAdd = true; //Signal that a new key was added.
       }
-
-      if(!bIsAdd)
-        bIsChange = true;
     }
 
+    if(!bIsAdd)
+      bIsChange = true;
+
     //Fire appropriate signal:
     if(bIsAdd)
     {
-        //Signal that a new key was added:
-        m_signal_user_added.emit(row_number);
+        //Signal that a new key was added"        
+        m_signal_user_added.emit(row);
     }
     else if(bIsChange)
     {
@@ -1237,7 +1143,7 @@ void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Gli
         {
           //Make sure that the entered data is suitable for this field type:
           bool success = false;
-          Glib::ustring text = get_value(row_number, model_column_index);
+          Glib::ustring text = get_value(row, model_column_index);
           Gnome::Gda::Value value = GlomConversions::parse_value(field_type, new_text, success);
           if(!success)
           {
@@ -1274,7 +1180,7 @@ void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Gli
         }
 
         if(do_signal)
-          m_signal_user_changed.emit(row_number, model_column_index);
+          m_signal_user_changed.emit(row, model_column_index);
       }
     }
   }
@@ -1315,20 +1221,6 @@ AddDel::type_signal_user_reordered_columns AddDel::signal_user_reordered_columns
   return m_signal_user_reordered_columns;
 }
 
-int AddDel::row_number_from_iterator(const Gtk::TreeModel::iterator iter)
-{
-  //This is a bit hacky. We should probably just use iterator in the interface.
-  int result = 0;
-
-  Gtk::TreeModel::Path path(iter);
-  typedef std::vector<int> type_vecInts;
-  type_vecInts vecIndices = path.get_indices();
-  if(!vecIndices.empty())
-    result = vecIndices[0];
-
-  return result;
-}
-
 Glib::ustring AddDel::string_escape_underscores(const Glib::ustring& text)
 {
   Glib::ustring result;
@@ -1358,24 +1250,25 @@ void AddDel::on_treeview_button_press_event(GdkEventButton* event)
     m_TreeView.get_path_at_pos((int)event->x, (int)event->y, path, pColumn, cell_x, cell_y);
     
     //Get the row:
-    Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
-    int tree_row = row_number_from_iterator(iter);
-
-    //Get the column:
-    int tree_col = 0;
-    int col_index = 0;
-
-    typedef std::vector<Gtk::TreeView::Column*> type_vecTreeViewColumns;
-    type_vecTreeViewColumns vecColumns = m_TreeView.get_columns();
-    for(type_vecTreeViewColumns::const_iterator iter = vecColumns.begin(); iter != vecColumns.end(); iter++)
+    Gtk::TreeModel::iterator iterRow = m_refListStore->get_iter(path);
+    if(iterRow)
     {
-      if(*iter == pColumn)
-        tree_col = col_index; //Found.
-        
-      col_index++;
-    }
+      //Get the column:
+      int tree_col = 0;
+      int col_index = get_count_hidden_system_columns();
+
+      typedef std::vector<Gtk::TreeView::Column*> type_vecTreeViewColumns;
+      type_vecTreeViewColumns vecColumns = m_TreeView.get_columns();
+      for(type_vecTreeViewColumns::const_iterator iter = vecColumns.begin(); iter != vecColumns.end(); iter++)
+      {
+        if(*iter == pColumn)
+          tree_col = col_index; //Found.
+
+        col_index++;
+      }
     
-    signal_user_activated().emit(tree_row, tree_col); 
+      signal_user_activated().emit(iterRow, tree_col);
+    }
   }
 
   on_button_press_event_Popup(event);
@@ -1407,7 +1300,7 @@ bool AddDel::on_treeview_column_drop(Gtk::TreeView* /* treeview */, Gtk::TreeVie
   return true;
 }
 
-guint AddDel::treeview_append_column(const Glib::ustring title, Gtk::CellRenderer& cellrenderer, const Gtk::TreeModelColumnBase& model_column, const Glib::ustring& column_id)
+guint AddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRenderer& cellrenderer, const Gtk::TreeModelColumnBase& model_column, const Glib::ustring& column_id)
 {
   TreeViewColumnGlom* pViewColumn = Gtk::manage( new TreeViewColumnGlom(title, cellrenderer) );
   pViewColumn->set_renderer(cellrenderer, model_column); //render it via the default "text" property.
@@ -1476,3 +1369,193 @@ void AddDel::set_auto_add(bool value)
 {
   m_auto_add = value;
 }
+
+Glib::RefPtr<Gtk::TreeModel> AddDel::get_model()
+{
+  return m_refListStore;
+}
+Glib::RefPtr<const Gtk::TreeModel> AddDel::get_model() const
+{
+  return m_refListStore;
+}
+
+bool AddDel::get_is_first_row(const Gtk::TreeModel::iterator& iter) const
+{
+  return iter == get_model()->children().begin();
+}
+
+bool AddDel::get_is_last_row(const Gtk::TreeModel::iterator& iter) const
+{
+  return iter == get_last_row();
+}
+
+Gtk::TreeModel::iterator AddDel::get_next_available_row_with_add_if_necessary()
+{
+  Gtk::TreeModel::iterator result;
+
+  if(!m_refListStore)
+    return result;
+
+  bool bPreventUserSignals = get_prevent_user_signals();
+  set_prevent_user_signals(true);
+
+  if(get_allow_user_actions()) //The extra blank line is only used if the user may add items:
+  {
+    Gtk::TreeModel::iterator iter = get_last_row();
+
+    if(iter != get_model()->children().end())
+    {
+      //Look at the last row:
+      if( get_is_placeholder_row(iter))
+      {
+        result = iter;
+      }
+      else
+      {
+        // The last line isn't blank, so we can not use it. Add another one.
+        result = m_refListStore->append();
+      }
+    }
+    else
+    {
+       // This is the first line.
+       result = m_refListStore->append();
+    }
+  }
+  else
+  {
+     result = m_refListStore->append(); //Add a new blank line. There are no blank lines.
+  }
+
+  set_prevent_user_signals(bPreventUserSignals);
+
+  return result;
+}
+
+Gtk::TreeModel::iterator AddDel::get_last_row() const
+{
+  //TODO_performance: Hopefully there is a better way to do this.
+  Gtk::TreeModel::iterator iter = get_model()->children().begin();
+  guint size = get_model()->children().size();
+  if(size > 1)
+  {
+    for(guint i = 0; i < (size -1); ++i)
+    {
+      ++iter;
+    }
+  }
+
+  return iter;
+}
+
+Gtk::TreeModel::iterator AddDel::get_last_row()
+{
+  //TODO_performance: Hopefully there is a better way to do this.
+  Gtk::TreeModel::iterator iter = get_model()->children().begin();
+  guint size = get_model()->children().size();
+  if(size > 1)
+  {
+    for(guint i = 0; i < (size -1); ++i)
+    {
+      ++iter;
+    }
+  }
+
+  return iter;
+}
+
+Glib::ustring AddDel::get_value_key(const Gtk::TreeModel::iterator& iter)
+{
+  return get_value(iter, m_col_key);
+}
+
+Gnome::Gda::Value AddDel::get_value_key_as_value(const Gtk::TreeModel::iterator& iter)
+{
+  bool parsed = false;
+  return GlomConversions::parse_value(m_key_field.get_glom_type(), get_value_key(iter), parsed);
+}
+
+
+void AddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Glib::ustring& strValue)
+{
+  if(!strValue.empty())
+  {
+    //This is not a placeholder anymore, if it every was:
+    iter->set_value(m_col_placeholder, false);
+  }
+  
+  iter->set_value(m_col_key, strValue);
+}
+
+bool AddDel::get_is_placeholder_row(const Gtk::TreeModel::iterator& iter) const
+{
+  if(!get_is_last_row(iter))
+    return false;
+
+  if(iter == m_refListStore->children().end())
+    return false;
+
+  bool val = false;
+  iter->get_value(m_col_placeholder, val);
+
+  return val;
+}
+
+bool AddDel::get_model_column_index(guint view_column_index, guint& model_column_index)
+{
+  //Initialize output parameter:
+  model_column_index = 0;
+
+  const guint hidden = get_count_hidden_system_columns();
+  const guint count = m_ColumnTypes.size();
+  if(hidden > count)
+    return false; //This should never happen.
+  
+  if(view_column_index >=  (count - hidden))
+    return false;
+
+  model_column_index = view_column_index + hidden; //There are 2 hidden columns at the start.
+
+  return true;
+}
+
+bool AddDel::get_view_column_index(guint model_column_index, guint& view_column_index)
+{
+  //Initialize output parameter:
+  view_column_index = 0;
+
+  if(model_column_index >=  m_ColumnTypes.size())
+    return false;
+    
+  if( !(m_ColumnTypes[model_column_index].m_visible) )
+    return false;
+
+  view_column_index = model_column_index - get_count_hidden_system_columns(); //There are 2 hidden columns at the start.
+
+  return true;
+}
+
+guint AddDel::get_count_hidden_system_columns()
+{
+ guint hidden_count = 0;
+  if(!(m_ColumnTypes[m_col_key].m_visible))
+    ++hidden_count;
+
+  if(!(m_ColumnTypes[m_col_placeholder].m_visible))
+    ++hidden_count;
+
+  return hidden_count;
+}
+
+void AddDel::set_rules_hint(bool val)
+{
+  m_TreeView.set_rules_hint(val);
+}
+
+void AddDel::set_key_type(const Field& field)
+{
+  m_key_field = field;
+}
+  
+      
+
